@@ -95,7 +95,7 @@ def denoise_bregman(binary_img):
 
     * binary_img (np.array): input array
 
-    Returns te denoised array
+    Returns the denoised array
     """
     thresf = np.array(binary_img, dtype=np.float32)
     #denoise = denoise_tv_bregman(thresf, weight=0.25)
@@ -229,13 +229,13 @@ def create_hocr_mask(img, mask_arr, hocr_word_data, downsample=None, dpi=None, t
             k = 0.1
             thres = threshold_image(np_lineimg, dpi, k)
             ones = np.count_nonzero(thres)
-            zero = (img.size[0] * img.size[1]) - ones
-            ratio = (ones/(zero+ones))*100
+            zero = np_lineimg.size - ones
+            ratio = (ones/(zero+ones))
 
             thres_invert = threshold_image(np_lineimg_invert, dpi, k)
             ones_i = np.count_nonzero(thres_invert)
-            zero_i = (img.size[0] * img.size[1]) - ones
-            inv_ratio = (ones_i/(zero_i+ones_i))*100
+            zero_i = np_lineimg.size - ones_i
+            inv_ratio = (ones_i/(zero_i+ones_i))
 
             if ratio < 0.3 or inv_ratio < 0.3:
                 th = None
@@ -467,7 +467,7 @@ def create_mrc_hocr_components(image, hocr_word_data,
 
 
 def encode_mrc_mask(np_mask, tmp_dir=None, jbig2=True, embedded_jbig2=False,
-                    timing_data=None):
+                    timing_data=None, debug=False):
     """
     Encode mask image either to JBIG2 or PNG.
 
@@ -494,10 +494,14 @@ def encode_mrc_mask(np_mask, tmp_dir=None, jbig2=True, embedded_jbig2=False,
     mask.save(mask_img_png, compress_level=0)
 
     if jbig2:
+        args = ['jbig2', mask_img_png]
         if embedded_jbig2:
-            out = subprocess.check_output(['jbig2', '-p', mask_img_png])
-        else:
-            out = subprocess.check_output(['jbig2', mask_img_png])
+            args = ['jbig2', '-p', mask_img_png]
+
+        if debug:
+            print('check_output: %s' % args, file=sys.stderr)
+
+        out = subprocess.check_output(args)
         fp= open(mask_img_jbig2, 'wb+')
         fp.write(out)
         fp.close()
@@ -512,7 +516,9 @@ def encode_mrc_mask(np_mask, tmp_dir=None, jbig2=True, embedded_jbig2=False,
 
 
 def encode_mrc_img(np_img, img_compression_flags, imgtype=None, tmp_dir=None,
-        jpeg2000_implementation=None, mrc_image_format=None, timing_data=None):
+        jpeg2000_implementation=None, mrc_image_format=None, timing_data=None,
+        threads=False,
+        debug=False):
     """
     Encode image as JPEG2000 or JPEG, with the provided compression settings
     and JPEG2000/JPEG encoder.
@@ -526,6 +532,7 @@ def encode_mrc_img(np_img, img_compression_flags, imgtype=None, tmp_dir=None,
     * jpeg2000_implementation (str): What JPEG2000 implementation to use
     * mrc_image_format (str): What image format to produce
     * timing_data (optional): Add time information to timing_data structure
+    * debug (bool, optional): Write debug info to stderr
 
     Returns the filepath to the JPEG2000 image
     """
@@ -548,14 +555,18 @@ def encode_mrc_img(np_img, img_compression_flags, imgtype=None, tmp_dir=None,
     if mrc_image_format == COMPRESSOR_JPEG:
         img.save(img_tiff, quality=100)
 
-        output = subprocess.check_output(['jpegoptim'] + img_compression_flags +
-                [img_tiff, '--stdout'])
+
+        args = ['jpegoptim'] + img_compression_flags + [img_tiff, '--stdout']
+        if debug:
+            print('check_output: %s' % args, file=sys.stderr)
+        output = subprocess.check_output(args)
         tmpfd=open(img_jp2, 'bw+') # XXX: FIXME: this defeats the point of a tmpfile
         tmpfd.write(output)
         tmpfd.close()
     else:
         encode_jpeg2000(img, img_jp2, jpeg2000_implementation,
-                        img_compression_flags, imgtype=imgtype)
+                        img_compression_flags, imgtype=imgtype,
+                        threads=threads, debug=debug)
 
 
     if timing_data is not None:
@@ -565,7 +576,8 @@ def encode_mrc_img(np_img, img_compression_flags, imgtype=None, tmp_dir=None,
 
 
 def encode_mrc_background(np_bg, bg_compression_flags, tmp_dir=None,
-        jpeg2000_implementation=None, mrc_image_format=None, timing_data=None):
+        jpeg2000_implementation=None, mrc_image_format=None, timing_data=None,
+        threads=None, debug=False):
     """
     Encode background image as JPEG2000, with the provided compression settings
     and JPEG2000 encoder.
@@ -583,11 +595,14 @@ def encode_mrc_background(np_bg, bg_compression_flags, tmp_dir=None,
     """
     return encode_mrc_img(np_bg, bg_compression_flags, 'bg', tmp_dir=tmp_dir,
             jpeg2000_implementation=jpeg2000_implementation,
-            mrc_image_format=mrc_image_format, timing_data=timing_data)
+            mrc_image_format=mrc_image_format, timing_data=timing_data,
+            threads=threads,
+            debug=debug)
 
 
 def encode_mrc_foreground(np_fg, fg_compression_flags, tmp_dir=None,
-        jpeg2000_implementation=None, mrc_image_format=None, timing_data=None):
+        jpeg2000_implementation=None, mrc_image_format=None, timing_data=None,
+        threads=False, debug=False):
     """
     Encode foreground image as JPEG2000, with the provided compression settings
     and JPEG2000 encoder.
@@ -605,13 +620,15 @@ def encode_mrc_foreground(np_fg, fg_compression_flags, tmp_dir=None,
     """
     return encode_mrc_img(np_fg, fg_compression_flags, 'fg', tmp_dir=tmp_dir,
             jpeg2000_implementation=jpeg2000_implementation,
-            mrc_image_format=mrc_image_format, timing_data=timing_data)
+            mrc_image_format=mrc_image_format, timing_data=timing_data,
+            threads=threads,
+            debug=debug)
 
 
 def encode_mrc_images(mrc_gen, bg_compression_flags=None, fg_compression_flags=None,
                       tmp_dir=None, jbig2=True, timing_data=None,
                       jpeg2000_implementation=None, mrc_image_format=None,
-                      embedded_jbig2=False):
+                      embedded_jbig2=False, threads=None, debug=False):
     mask_img_jbig2, mask_img_png = encode_mrc_mask(next(mrc_gen),
             tmp_dir=tmp_dir, jbig2=jbig2, embedded_jbig2=embedded_jbig2,
             timing_data=timing_data)
@@ -620,7 +637,7 @@ def encode_mrc_images(mrc_gen, bg_compression_flags=None, fg_compression_flags=N
     fg_img_jp2 = encode_mrc_foreground(np_fg, fg_compression_flags, tmp_dir=tmp_dir,
                                        jpeg2000_implementation=jpeg2000_implementation,
                                        mrc_image_format=mrc_image_format,
-                                       timing_data=timing_data)
+                                       timing_data=timing_data, threads=threads, debug=debug)
     fg_h, fg_w = np_fg.shape[0:2]
     np_fg = None
 
@@ -628,7 +645,7 @@ def encode_mrc_images(mrc_gen, bg_compression_flags=None, fg_compression_flags=N
     bg_img_jp2 = encode_mrc_background(np_bg, bg_compression_flags, tmp_dir=tmp_dir,
                                        jpeg2000_implementation=jpeg2000_implementation,
                                        mrc_image_format=mrc_image_format,
-                                       timing_data=timing_data)
+                                       timing_data=timing_data, threads=threads, debug=debug)
     bg_h, bg_w = np_bg.shape[0:2]
     np_bg = None
 
